const path = require('path');

const customImportResolver = path.resolve(
  path.join(__dirname, './lib/eslint_import_resolver')
);

module.exports = {
  root: true,
  parser: '@babel/eslint-parser',
  extends: ['airbnb', 'plugin:import/warnings', 'plugin:jest/recommended'],
  plugins: ['react', 'import', 'jest', 'jsx-a11y', 'webdriverio'],
  env: {
    browser: true,
    commonjs: true,
    es6: true,
    jest: true,
    // wdio specs use mocha
    mocha: true,
    node: true,
    'webdriverio/wdio': true,
  },
  rules: {
    // http://eslint.org/docs/rules/
    'arrow-parens': ['error', 'always'],
    'comma-dangle': [
      'error',
      {
        arrays: 'always-multiline',
        objects: 'always-multiline',
        imports: 'always-multiline',
        exports: 'always-multiline',
        functions: 'never',
      },
    ],
    'function-paren-newline': 'off',
    'global-require': 'off',
    'max-len': [
      'error',
      80,
      2,
      {
        ignoreUrls: true,
        ignoreComments: false,
        ignoreRegExpLiterals: true,
        ignoreStrings: true,
        ignoreTemplateLiterals: true,
      },
    ],
    'no-underscore-dangle': 'off',
    'no-unused-expressions': [
      'error',
      {
        allowShortCircuit: false,
        allowTernary: false,
        allowTaggedTemplates: true,
      },
    ],
    'object-curly-newline': 'off',
    'padding-line-between-statements': [
      'error',
      {
        blankLine: 'always',
        prev: '*',
        next: ['if', 'return'],
      },
      {
        blankLine: 'always',
        prev: 'if',
        next: '*',
      },
      {
        blankLine: 'always',
        prev: ['const', 'let', 'var'],
        next: '*',
      },
      {
        blankLine: 'any',
        prev: ['const', 'let', 'var'],
        next: ['const', 'let', 'var'],
      },
    ],

    // https://github.com/benmosher/eslint-plugin-import/blob/master/docs/rules
    'import/no-dynamic-require': 'warn',
    // Turning off dependency checking until monorepos are supported:
    // https://github.com/benmosher/eslint-plugin-import/issues/458
    'import/no-extraneous-dependencies': 'off',
    'import/no-named-as-default': 'off',

    // https://github.com/evcohen/eslint-plugin-jsx-a11y/tree/master/docs/rules
    'jsx-a11y/anchor-is-valid': 'warn',
    'jsx-a11y/click-events-have-key-events': 'off',
    'jsx-a11y/media-has-caption': 'off',
    'jsx-a11y/mouse-events-have-key-events': 'off',
    'jsx-a11y/no-static-element-interactions': 'off',

    // https://github.com/yannickcr/eslint-plugin-react/tree/master/docs/rules
    'react/jsx-filename-extension': 'off',
    'react/jsx-wrap-multilines': 'off',

    // http://eslint.org/docs/rules/valid-jsdoc
    'valid-jsdoc': [
      'error',
      {
        preferType: {
          Boolean: 'boolean',
          Number: 'number',
          String: 'string',
          array: 'Array',
          function: 'Function',
          object: 'Object',
          promise: 'Promise',
        },
      },
    ],
  },
  settings: {
    'import/resolver': {
      [customImportResolver]: {
        moduleNameMapper: {
          '^types/(.*)': './types/$1',
        },
      },
    },
  },
};
